__all__ = [
    "french_wind_turbine_analysis",
    "wind_pca",
    "plot_wind_pca",
    "vre_computation",
    "vre_aggregation",
]

DIC_OFFSHORE = {
           "200": "Calv",
           "201": "Dunk",
           "202": "Feca",
           "203": "Lion",
           "204": "Groix",
           "205": "Grui",
           "206": "Trep",
           "207": "Mist",
           "208": "Noir",
           "209": "Prov",
           "210": "StBr"
    }

DIC_NAME_OFFSHORE = {
                   "Calvados": "Calv",
                   "Dunkerque Offshore": "Dunk",
                   "Fécamp Offshore": "Feca",
                   "Golfe du Lion": "Lion",
                   "Groix": "Groix",
                   "Gruissan": "Grui",
                   "Le Tréport": "Trep",
                   "Mistral": "Mist",
                   "Noirmoutier": "Noir",
                   "Provence Grand Large": "Prov",
                   "Saint-Brieuc": "StBr"
    }


DIC_REGION = {
    "Rhone Alpes": [
        "01",
        "03",
        "07",
        "15",
        "26",
        "38",
        "42",
        "43",
        "63",
        "73",
        "74",
    ],
    "Bourgogne Franche Comte": ["21", "25", "39", "58", "70", "71", "89", "90"],
    "Bretagne": ["22", "29", "35", "56"],
    "Centre Val de Loire": ["18", "28", "36", "37", "41", "45"],
    "Grand Est": ["08", "10", "51", "52", "54", "55", "57", "67", "68", "88"],
    "Haut de France": ["02", "59", "60", "62", "80"],
    "Ile de France": ["75", "77", "78", "91", "92", "93", "94", "95"],
    "Normandie": ["14", "27", "50", "61", "76"],
    "Nouvelle Aquitaine": [
        "16",
        "17",
        "19",
        "23",
        "24",
        "33",
        "40",
        "47",
        "64",
        "79",
        "86",
        "87",
    ],
    "Occitanie": [
        "09",
        "11",
        "12",
        "30",
        "31",
        "32",
        "34",
        "46",
        "48",
        "65",
        "66",
        "81",
        "82",
    ],
    "Pays de Loire": ["44", "49", "53", "72", "85"],
    "PACA": ["04", "05", "06", "13", "83", "84"],
    "OFFSHORE": [str(x) for x in range(200,221)]
}

MONTHS =  [
                "Jan",
                "Feb",
                "Mar",
                "Apr",
                "May",
                "Jun",
                "Jul",
                "Aug",
                "Sep",
                "Oct",
                "Nov",
                "Dec",
            ]    

EOLES_ONSHORE = {"manufacturer" :"Vestas",
                 "turbine" : "V80/2000",
                 "hub_height" : 80}
EOLES_OFFSHORE = {"manufacturer" :"Siemens",
                 "turbine" : "SWT-4.0-130",
                 "hub_height" : 120}

COL_NAMES = [
        "Latitude",
        "Longitude",
        "Total power",
        "Manufacturer",
        "Model",
        "Hub height",
    ]
 

def french_wind_turbine_analysis(year, root_vwf, root_county, mode="max", EOLES=False):

    """
    This routine for a selected year :
        - plots the french counties using the contours extracted from depfr-txt.zip 
        downloaded from  https://melusine.eu.org/syracuse/jms/depfr/ 
        - plots the location (red dots) of the french wind farms using the WindPower file Windfarms_France_20201220.xls
        - plots for each county the location (blue star) of the most powerful wind farm.
        - builds a dataframe df with 
        index county Insee code | Latitude |Longitude|Total power|Manufacturer|Model|Hub height|
        where Latitude, Longitude, Total power, Manufacturer, Model, Hub height are given for the largest file in the county
    
    Args:
        year (int): the working year
        root_vwf (path): root of the WindPower file Windfarms_France_20201220.xls
        root_county (path): root of county contours downloaded from https://melusine.eu.org/syracuse/jms/depfr/ . 
          Download depfr-txt.zip (1.23 Mo – 6 juin 2010), put the unzipped files in yout Path home/depfr-txt
        mode (string): mode = 'max' in each county we select the wind farm with the greatest power capacity
                       mode = 'min' in each county we select the wind farm with the lowest power capacity
                       mode = 'median' in each county we select the wind farm with the median power capacity
                       (default = 'max')
        EOLES (boolean) : if True we use EOLES condition hub height = 80, tubine = Vestas V80 2000
                       
    Returns:
        df (dataframe): countyInsee code|Latitude|Longitude|Total power|Manufacturer|Model|Hub height| for the french metropolitan wind farm
        median_hub_height (float): median value of the hubs height
        df_offshore (dataframe): countyInsee code|Latitude|Longitude|Total power|Manufacturer|Model|Hub height| for the french offshore wind farm
    """

    # Standard library imports
    from pathlib import Path
    import os
    import re

    os.environ["PROJ_LIB"] = str(
        Path.home() / Path("Anaconda3/pkgs/proj4-5.2.0-ha925a31_1/Library/share")
    )

    # 3rd party imports
    import matplotlib.pyplot as plt
    import pandas as pd
    import numpy as np
    from mpl_toolkits.basemap import Basemap
    import warnings

    warnings.filterwarnings("ignore")
    
    extract_year = lambda x: int(x.split("/")[0]) if "/" in x else int(x)
    

    def lim_departement(dep,map_french):

        """
        parse the files ##-nom_departement.txt 
        where xx is the departement identifier (ex Isère ## = 38)
        
        Args:
            dep (string): county file name 
            map_french (object)
            
        """

        v = root_county / Path(dep)
        file = open(v, "r", encoding="utf8")
        lines = file.readlines()
        file.close()

        lat = []  # list of county contor latitudes
        lon = []  # list of county contor longitudes
        
        for i, y in enumerate(lines[9].split(",")[0:-1]):  # skip 9 lines
            if i % 2 == 1:
                lon.append(float(y))
            elif i == 0:  # corection of a bug in the txt file
                lat.append(float(y))
            else:
                lat.append(float(y[2:]))
                
        x, y = map_french(lat, lon)
        return x, y, map_french(*[float(x) for x in lines[2].split(",")[0:-1]])
    
        
    def plot_french_wind_farm():
        
        # draw the France map
        map_french = Basemap(
                             urcrnrlat=51.5,
                             llcrnrlat=42,
                             urcrnrlon=9,
                             llcrnrlon=-5.5,
                             suppress_ticks=True,
                             rsphere=(6378137.00, 6356752.3142),
                             resolution="h",
                             projection="merc",
                             lat_0=(51.5 + 42.0) / 2,
                             lon_0=(9 - 5.5) / 2,
                        )

        total_power = []
        
        for county_file in os.listdir(root_county): # list of french counties formatted as ##_CountyNane.txt
                                          #  ex 38_isere.txt
            insee_code_county = county_file[0:2]
            if (
                (insee_code_county != "2A") and # exclude Corse du Sud
                (insee_code_county != "2B") and # exclude Haute Corse
                int(insee_code_county) < 97     # exclude overseas counties
            ):  

                x, y, c = lim_departement(county_file,map_french)
                plt.plot(
                    x,
                    y,
                    color="k",
                    alpha=0.7,
                    linewidth=1,
                    solid_capstyle="round",
                    zorder=60,
                )

                try:  # the county is equipped with wind farm
                    total_power.append(d[insee_code_county])
                    prod = round(100 * d[insee_code_county] / french_total_power, 2)
                except:
                    total_power.append(0)
                    prod = 0

                ax = plt.gca()  # get current axes instance
                if int(insee_code_county) <= 90:
                    ax.text(
                        c[0],
                        c[1],
                        str(prod) + "%",
                        fontsize=12,
                        fontweight="bold",
                        ha="center",
                        va="top",
                        color="k",
                    )


        # plot wind farm locations
        w_onshore = wind_farm.query(
            '(Status == "Production")  &  \
                                      (Area != "Offshore")     &  \
                                      (Latitude !="#ND")       &  \
                                      (Longitude !="#ND")      &  \
                                      (`Total power` !="#ND")'
        )

        w_onshore = w_onshore.query(
            "(Latitude > 42)  &  (Latitude < 52) &  (Longitude < 14)"
        )

        lat = np.array(w_onshore["Latitude"])
        long = np.array(w_onshore["Longitude"])
        pw = (np.array(w_onshore["Total power"]) / 500).astype(int)
        x, y = map_french(long, lat)
        plt.scatter(x, y, s=pw, c="r")

        # plot location of the selected farm per county
        long, lat = zip(*list(gps_selected_farm.values()))
        long = list(long)
        lat = list(lat)
        x, y = map_french(lat, long)
        plt.scatter(x, y, s=50, c=color_selected_farm, marker="*", alpha=0.8)

        # plot offshore wind farm location in production
        w_offshore = wind_farm_offshore.query(
            '(Status == "Production") &  \
                                               (Area=="Offshore")'
        )
        w_offshore = w_offshore.query(
            "(Latitude > 42)  &  (Latitude < 52) &  (Longitude < 14)"
        )
        lat = np.array(w_offshore["Latitude"])
        long = np.array(w_offshore["Longitude"])
        x, y = map_french(long, lat)
        _ = plt.scatter(x, y, s=100, c="r")

        # plot offshore wind farm location planned
        w_offshore = wind_farm_offshore.query(
            '(Status == "Approved") &  \
                                               (Area=="Offshore")'
        )
        w_offshore = w_offshore.query(
            "(Latitude > 42)  &  (Latitude < 52) &  (Longitude < 14)"
        )
        lat = np.array(w_offshore["Latitude"])
        long = np.array(w_offshore["Longitude"])
        pw = (np.array(w_offshore["Total power"]) / 500).astype(int)
        x, y = map_french(long, lat)
        _ = plt.scatter(x, y, s=0.8 * pw, c="g")
        
        for x_y_name in zip(x,y,[DIC_NAME_OFFSHORE[x] for x in w_offshore["Name"]]):
            plt.annotate(x_y_name[2],(x_y_name[0],x_y_name[1]))
            
        
        _ = plt.title(
                f"year = {year}, \
                        total power = {french_total_power/1.e6:.2f} Gw, \
                        number of farms = {len(wind_farm)}, \
                        median hub height = {median_hub_height:.2f} m  \
                        {per_cent_height:.1f}% of known hub heigth \n  \
                        selection mode : {mode},  number of equipped counties : {len(df_onshore)}"
                )
        plt.axis("off")
        
        return

    
    ##########################################
    # read the wind farm data file and select
    #
    ##########################################
    
    wind_farm = pd.read_excel(
        root_vwf, sheet_name="Windfarms", parse_dates=["Commissioning date"]
    )

    wind_farm.drop(0, axis=0, inplace=True)
    wind_farm_offshore = wind_farm.query(
        'Status=="Approved" &  \
                                          Area=="Offshore"'
    )
    wind_farm = wind_farm.query('`Commissioning date` != "#ND"')
    wind_farm["year"] = wind_farm["Commissioning date"].apply(extract_year)
    wind_farm = wind_farm.query("year <= @year")

    # build the dictionary d = {'county Insee code : nominal total power of the most powerfull county wind farm}
    d = {}
    gps_selected_farm = {}
    selected_farm = {}
    for (
        x
    ) in (
        wind_farm.Area.unique()
    ):  # look for all different counties equiped with wind farm

        if ("Offshore" not in x) & (
            "2B" not in x
        ):  # eliminate offshore and retain french metropolitan county

            num_county = re.findall("\d{1,3}", x)[0]

            wind_farm_county = wind_farm.query(
                '(`Total power` !="#ND") &    \
                                                ( Status == "Production") &  \
                                                (Area==@x)'
            )

            total_power = wind_farm_county["Total power"].sum()

            d[num_county] = total_power

            if int(num_county) < 96:  # select the largest county wind farm

                if mode == "max":
                    idx_pow_max = wind_farm_county["Total power"].astype(float).argmax()
                    idx_pow_index = wind_farm_county.index[idx_pow_max]
                    color_selected_farm = "b"
                elif mode == "min":
                    idx_pow_min = wind_farm_county["Total power"].astype(float).argmin()
                    idx_pow_index = wind_farm_county.index[idx_pow_min]
                    color_selected_farm = "k"
                elif mode == "median":
                    wind_farm_median_pow = np.percentile(
                        wind_farm_county["Total power"], 50, interpolation="nearest"
                    )
                    idx_pow_median = list(wind_farm_county["Total power"]).index(
                        wind_farm_median_pow
                    )
                    idx_pow_index = wind_farm_county["Total power"].index[
                        idx_pow_median
                    ]
                    color_selected_farm = "g"

                gps_selected_farm[num_county] = (
                    wind_farm_county.loc[idx_pow_index, "Latitude"],
                    wind_farm_county.loc[idx_pow_index, "Longitude"],
                )

                if EOLES:  # we choose EOLES conditions
                    selected_farm[num_county] = (
                        wind_farm_county.loc[
                            idx_pow_index, "Latitude"
                        ],  # Latitude of the largest county wind farm
                        wind_farm_county.loc[
                            idx_pow_index, "Longitude"
                        ],  # Longitude of the largest county wind farm
                        total_power,
                        EOLES_ONSHORE["manufacturer"] ,
                        EOLES_ONSHORE["turbine"],
                        EOLES_ONSHORE["hub_height"],
                    )  # Hub height of the largest county wind farm

                else:

                    selected_farm[num_county] = (
                        wind_farm_county.loc[
                            idx_pow_index, "Latitude"
                        ],  # Latitude of the largest county wind farm
                        wind_farm_county.loc[
                            idx_pow_index, "Longitude"
                        ],  # Longitude of the largest county wind farm
                        total_power,
                        wind_farm_county.loc[
                            idx_pow_index, "Manufacturer"
                        ],  # Manufacturer of the largest county wind farm
                        wind_farm_county.loc[
                            idx_pow_index, "Turbine"
                        ],  # Turbine of the largest county wind farm
                        wind_farm_county.loc[idx_pow_index, "Hub height"],
                    )  # Hub height of the largest county wind farm

    french_total_power = sum(d.values())
    
    # build onshore farms data frame df
    df_onshore = pd.DataFrame.from_dict(selected_farm).T
    df_onshore.columns = COL_NAMES
    
    hub_height = [float(x) for x in df_onshore["Hub height"] if x != "#ND"]
    per_cent_height = 100 * len(hub_height) / len(df_onshore["Hub height"])
    median_hub_height = np.median(hub_height)
    
    

    # build offshore farms data frame df_offshore
    farm_offshore = {}
    w_offshore = wind_farm_offshore.query('(Area=="Offshore")')
    w_offshore = w_offshore.query(
        "(Latitude > 42)  &  (Latitude < 52) &  (Longitude < 14)"
    )
    for idx in w_offshore.index:
        farm_offshore[w_offshore.loc[idx, "Name"]] = [
            w_offshore.loc[idx, "Latitude"],
            w_offshore.loc[idx, "Longitude"],
            w_offshore.loc[idx, "Total power"],
            EOLES_OFFSHORE["manufacturer"] ,
            EOLES_OFFSHORE["turbine"],
            EOLES_OFFSHORE["hub_height"],
        ]
        
    df_offshore = pd.DataFrame.from_dict(farm_offshore).T
    df_offshore.columns = COL_NAMES
    df_offshore.index = [str(x) for x in (200 + np.arange(0, len(df_offshore), 1))]
    
    fig = plt.figure(figsize=(15, 15))
    plot_french_wind_farm()
    plt.show()

    return df_onshore, median_hub_height, df_offshore


def wind_pca(X):

    import numpy as np
    import matplotlib.pyplot as plt

    # Build a square symetric correlation matrix of rank  N_var
    Cor = np.dot(X, X.T)

    # Compute the eigenvalues and eigenvectors
    lbd, Eigen_vec = np.linalg.eig(Cor)
    # print(np.matmul(Cor,Eigen_vec)-np.matmul(Eigen_vec,np.eye(N_var)*lbd))

    # sort by decreasing value of eigenvalues
    w = sorted(list(zip(lbd, Eigen_vec.T)), key=lambda tup: tup[0], reverse=True)
    vp = np.array([x[0] for x in w])
    L = np.array([x[1] for x in w]).reshape(np.shape(Eigen_vec)).T

    F = np.real(np.matmul(X.T, L))
    Eigen_vec = np.real(Eigen_vec)

    labels = ["PC" + str(x) for x in range(1, 10)]
    plt.bar(x=range(1, 10), height=vp[0:9], tick_label=labels, color="turquoise")
    plt.title("Scree plot")
    return F


def plot_wind_pca(F, df, wind, year):

    '''Plot the PCA results (component PC2 versus component PC1)
    
    Args:
        F (nparray): wind matrix NxM (N county, M time samples)
        df (data frame): generated by
        F (nparray): PCA load matrix 
        year (int) year processed
    '''

    # Standard library imports
    import re 
    
    # 3rd party imports
    import matplotlib.pyplot as plt
    from adjustText import adjust_text
    
    idx_county = lambda county : re.findall("^\d{1,3}", county)[0]
    county_name = lambda county: county if int(county )<200  else DIC_OFFSHORE[county]
    
    txt_optimisation = True
    
    
    dic_color = {
        "Rhone Alpes": "b",
        "Bourgogne Franche Comte": "r",
        "Bretagne": "y",
        "Centre Val de Loire": "c",
        "Grand Est": "g",
        "Haut de France": "m",
        "Ile de France": "gray",
        "Normandie": "k",
        "Nouvelle Aquitaine": "orange",
        "Occitanie": "gold",
        "Pays de Loire": "olive",
        "PACA": "aqua",
        "OFFSHORE": "r",
    }
    inv = {}
    for x, y in DIC_REGION.items():
        for k in y:
            inv[k] = dic_color[x]
    c = []
    for county in wind.columns:
        c.append(inv[idx_county(county)])   
  
    plt.figure(figsize=(15, 15))

    idx_dep = {idx_county(x): i for i, x in enumerate(wind.columns)}
    for region in DIC_REGION.keys():
        idx = [
            idx_dep[idx_county(x)] for x in wind.columns if idx_county(x) in DIC_REGION[region]
        ]

        X = [F[i][0] for i in idx]
        Y = [F[i][1] for i in idx]

        plt.scatter(
            X,
            Y,
            c=dic_color[region],
            s=(df.T.values[2][idx] / 7000).astype(int),
            label=region,
            alpha=0.6,
        )

    plt.legend(loc=5, bbox_to_anchor=(0.3, 0.5))
    plt.xlabel("PC1")
    plt.ylabel("PC2")
    plt.title(f"Wind PCA analysis ({year})")

    if not txt_optimisation:
        for idx, sample in enumerate(wind.columns):
            plt.annotate(idx_county(sample), (F[:, 0][idx], F[:, 1][idx]), size=15, c=c[idx])

    else:
        texts = []
        for idx, sample in enumerate(wind.columns):
            texts.append(
                plt.text(
                    F[:, 0][idx],
                    F[:, 1][idx],
                    county_name(idx_county(sample)), 
                    size=11,
                    fontweight="bold",
                    c=c[idx],
                )
            )
        adjust_text(texts, save_steps=False)


def vre_aggregation(year, root_outputs, root, new_turbine=False):

    """
    This routine aggregates all the excel files in three files:
        - the first contains the loading rate matrix MxN with 8760 rows (number of hours in a year) and
          N columns corresponfing the the counties
        - the second contains the wind speed matrix MxN with 8760 rows (number of hours in a year) and
          N columns corresponfing the the counties
        - the third contains, for each county, the meta parameters provided to the ninja api.
        
    Args:
        - year (int): The current year 
        - root_outputs (string): the root containing the files to aggregates
    """

    # Standard library imports
    from pathlib import Path
    import os
    import re

    # 3rd party imports
    import pandas as pd
    from scipy.interpolate import interp1d

    #  interpolation
    if new_turbine:
        df_turbine = pd.read_excel(
            root / Path("Vestas_V110_2000_w02.xlsx"), index_col="Unnamed: 0"
        )
        wind_speed = df_turbine["speed"].values
        power = df_turbine["power"].values
        f = interp1d(wind_speed, power, kind="cubic")

    for idx, file in enumerate(os.listdir(root_outputs)):
        sub_str = re.findall(
            "^\d{1,3}.xlsx", file
        )  # filter the files with the right format ##.xlsx

        if len(sub_str):  # is a valid file found?
            num_county = sub_str[0].split(".xlsx")[
                0
            ]  # county Insee code (ex 38 for Isère) or 2## for offshore
            dg = pd.read_excel(
                root_outputs / Path(sub_str[0]), index_col="Unnamed: 0"
            )  # store the vre
            dg.columns = [
                str(num_county),
                str(num_county) + "_wind",
            ]  # store the wind speed

            if new_turbine:
                pow_interp = f(dg[str(num_county) + "_wind"])
                dg[str(num_county)] = pow_interp

            if idx == 0:
                vre_new = dg[str(num_county)]
                wind = dg[str(num_county) + "_wind"]
            else:
                vre_new = pd.merge(
                    vre_new, dg[str(num_county)], right_index=True, left_index=True
                )
                wind = pd.merge(
                    wind,
                    dg[str(num_county) + "_wind"],
                    right_index=True,
                    left_index=True,
                )

    dict_metadata = {}
    for idx, file in enumerate(os.listdir(root_outputs)):
        sub_str = re.findall("^\d{1,3}_metadata.xlsx", file)
        if len(sub_str):
            num_county = sub_str[0].split(".xlsx")[
                0
            ]  # county Insee code (ex 38 for Isère) or 2## for offshore
            dg = pd.read_excel(root_outputs / Path(sub_str[0]), index_col="Unnamed: 0")
            dict_metadata[num_county] = [
                dg.loc["params", "lat"],
                dg.loc["params", "lon"],
                dg.loc["params", "date_from"],
                dg.loc["params", "date_to"],
                dg.loc["params", "height",],
                dg.loc["params", "turbine"],
            ]

    df_metadata = pd.DataFrame.from_dict(dict_metadata)
    df_metadata.index = ["lat", "lon", "date_from", "date_to", "height", "turbine"]

    return vre_new, wind, df_metadata


def vre_computation(year, vre_new, df, root_inputs_eoles, wind_farm_type="onshore"):

    # Standard library import
    import datetime

    # Local import
    from .vre_eoles_read import read_LF_CIRED
    from .vre_eoles_read import read_parc_eolien
    from .vre_eoles_read import read_rte
    

    # 3rd party import
    import numpy as np
    import pandas as pd
    import matplotlib.pyplot as plt

    fe = read_LF_CIRED(year, wind_farm_type, root_inputs_eoles)

    per_cent_power_county = df["Total power"].values.astype(float)
    per_cent_power_county = per_cent_power_county / sum(per_cent_power_county)

    # computation of the load time at the France scale
    VRE = np.sum(
        vre_new.values.astype(float) * per_cent_power_county, axis=1
    )  # beware the broadcasting

    deb = datetime.datetime(year, 1, 1)
    temps = [deb + datetime.timedelta(hours=i) for i in range(int(len(VRE)))]
    VRE = pd.DataFrame(VRE, index=temps, columns=["vre"])
    VRE = pd.merge(VRE, fe, right_index=True, left_index=True)

    if wind_farm_type == "onshore":
        P_eol = read_parc_eolien()  # build rte vre
        vre_rte = read_rte(year)
        vre_rte = vre_rte["Eolien"] / P_eol[year]
        VRE = pd.merge(VRE, vre_rte, right_index=True, left_index=True)
        VRE.columns = ["This sudy", "EOLES", "RTE"]
        ax = VRE.resample("M").mean().plot(kind="bar")
        _ = ax.set_xticklabels(
            MONTHS,
            rotation=0,
            fontsize=12,
        )
        plt.ylabel("Capacity Factor")
        plt.title(
            f"Onshore, year : {year} \n  \
                  This sudy = {100*VRE['This sudy'].mean():.1f}% RTE = {100*VRE['RTE'].mean():.1f}% EOLES = {100*VRE['EOLES'].mean():.1f}%"
        )
        plt.legend(loc=9)
    else:
        VRE.columns = ["This sudy", "EOLES"]
        ax = VRE.resample("M").mean().plot(kind="bar")
        _ = ax.set_xticklabels(
            MONTHS,
            rotation=0,
            fontsize=12,
        )
        plt.title(
            f"Offshore, year : {year} \n  \
                  This sudy = {100*VRE['This sudy'].mean():.1f}% EOLES = {100*VRE['EOLES'].mean():.1f}%"
        )
        plt.ylabel("Capacity Factor")
        plt.legend(loc=9)

    return VRE



__all__ = [
    "weighted_polynomial_reg",
    "extrapolate_power_law",
    "gaussian_smoothing",
    "conv_vwf",
    "polynomial",
]


def weighted_polynomial_reg(X, Y, w, degree=1):

    # 3rd party import
    import numpy as np
    from sklearn.preprocessing import PolynomialFeatures, normalize
    from sklearn.linear_model import LinearRegression

    X = X.reshape(-1, 1)
    Y = Y.reshape(-1, 1)
    w = w.reshape(-1, 1)

    poly_reg = PolynomialFeatures(degree=degree)
    X_poly = poly_reg.fit_transform(X)  # Vandermonde Matrix

    # Solve Weighted Normal Equation
    A = np.linalg.inv(X_poly.T @ (w * X_poly))
    beta = (A @ X_poly.T) @ (w * Y)

    return beta


def polynomial(x, coeff):
    #Define Polynomial - Use Numpy for optimization
    y = 0
    for p, c in enumerate(coeff):
        y += c * x**p
    return y

def extrapolate_power_law(h, v):

    """
    h = list of heights for which we have measurements
    v = list of speeds at those heights
    hx = hub height at which you want to know the speed
    """

    # 3rd party import
    import numpy as np

    log_h = np.log(h)

    weights = [1, 1, 2]

    # linearise and perform a ls fit
    # weight the data at 50m more strongly
    log_h = np.log(h)

    fit = weighted_polynomial_reg(log_h, v, weights, degree=1)

    # extract our coefficients
    # v2 / v1 =  (h2 / h1) ^ alpha, therefore v2 = epsilon * h ^ alpha
    epsilon = np.exp(fit[0])
    alpha = fit[1]

    return epsilon, alpha


def gaussian_smoothing(mu, sig, x, y):

    """translation of the function convoluteFarmCurve = function(myCurve,
    myMean, mySD) ( VWF-EXTRAS.R) from R to Python.
    
    Args:
        mu (float) : gauss filter mean value
        sig (float) : gauss filter sigma
        x (nparray) : weed speed values
        y (nparray) : Power response
    Returns:
        y_convolve (np.array) : smoothed power curve
    
    """
    import numpy as np

    def gaussian(x, mu, sig):
        return (
            1.0
            / (np.sqrt(2.0 * np.pi) * sig)
            * np.exp(-np.power((x - mu) / sig, 2.0) / 2)
        )

    # zero padding
    x_filter = np.arange(-10, 45.01, 0.01)
    y_filter = np.concatenate((np.zeros(1000), y, np.zeros(1000)))

    # gaussian convolution
    w = 10
    convolver_x = np.arange(-w, w + 0.01, 0.01)
    convolver_y = gaussian(convolver_x, mu, sig)
    convolver_y = convolver_y / sum(convolver_y)
    y_convolve = np.convolve(y_filter, convolver_y, mode="same")
    y_convolve = y_convolve[
        min(np.argwhere(x_filter >= 0))[0] : min(np.argwhere(x_filter >= 35))[0]
    ]
    x_filter = x_filter[
        min(np.argwhere(x_filter >= 0))[0] : min(np.argwhere(x_filter >= 35))[0]
    ]

    return x_filter, y_convolve


def conv_vwf(ws, alpha, beta, power):

    """
    Smoothing procedure of a power curve using the procedure descibed in :
    https://ars.els-cdn.com/content/image/1-s2.0-S0360544216311811-mmc1.pdf
    
    Args:
        ws (nparray): a list of wind speed values w
        alpha (float): sigma = alpha * w + beta
        beta (float): sigma = alpha * w + beta
        power (nparray): manufacturer power curve (the wind speed sample interval is 
                        supposed to be equal to 0.01 m/s)
        
    Returns:
        power_conv (nparray): convolution of the power curve computed at at speed values ws
    """

    # Standard library import
    from collections import deque

    # 3rd party imports
    import numpy as np

    delta_w = 0.01  # should be an argument

    speed_widening = 10  # m/s
    n_zero_padd = int(speed_widening / delta_w)  # must be even

    power_conv = []
    for w in ws:
        sigma_square = beta + alpha * w  # note we will use sigma^2
        n_sigma_square = 4

        # kernel support we take int(sigma_square+1) to take care of sigma_square<1 which
        # would result in a zero size support
        kernel_support = np.arange(
            -int(n_sigma_square * int(sigma_square + 1)),
            int(n_sigma_square * int(sigma_square + 1)) + delta_w,
            delta_w,
        )  # m/s
        len_kernel = len(kernel_support)
        idx_max_gauss = int(len_kernel / 2)

        # kernel computation on [-n_sigma_square*sigma_square, +n_sigma_square*sigma_square]
        # with a step of delta_w (m/s)
        kernel = (
            1.0
            / np.sqrt(2.0 * np.pi * sigma_square)
            * np.exp(-np.power(kernel_support, 2.0) / (2 * sigma_square))
        )

        # power curve left and right zero padding
        power_ = np.concatenate((np.zeros(n_zero_padd), power, np.zeros(n_zero_padd)))
        len_power = len(power_)

        # we truncate or right padd the kernel with 0 so that len(kernel)=len(power)
        if len_kernel < len_power:
            kernel = np.concatenate((kernel, np.zeros(len_power - len_kernel)))
        else:
            kernel = kernel[0:len_power]

        # we righ rotate the kernel so that the location of its max is
        # at w + n_zero_padd * delta_w the shifted value of w
        kernel = deque(kernel)
        kernel.rotate(int((w + n_zero_padd * delta_w) / delta_w - idx_max_gauss))
        kernel = np.array(list(kernel))

        # we compute the convolution
        power_conv.append(np.dot(power_, kernel) * delta_w)

    return power_conv



__all__ = [
    "stat_prod_electricite",
    "stat_prod_consommation",
    "prod_eolien_evolution_par_jour",
]




def stat_prod_electricite(year):

    
    #Local import
    from .vre_eoles_read import read_rte
    
    # 3rd party import
    import matplotlib.pyplot as plt
    
    
    format_date = lambda x : f'{x.year}/{x.month}/{x.day} à {x.hour}h'
    B = read_rte(year)
    fig=plt.figure(figsize=(10,10))
    B[["Fioul","Charbon","Gaz","Nucléaire","Eolien","Solaire","Hydraulique","Bioénergies"]] .sum().plot(kind='pie',fontsize=18)
    plt.ylabel("")
    plt.title(f"{year}")
    plt.show()
    
    cat = ["Fioul","Charbon","Gaz","Nucléaire","Eolien","Solaire","Hydraulique","Bioénergies"]
    print(f"Consommation et production d'énergie électrique pour l'année {year}\n")
    print(f"Consommation : {B['Consommation'].sum()/1.e6} TWh")
    prod = sum([x for x in B[cat] .sum()])
    for x in [(x[0],x[1]) for x in zip(cat,B[cat].sum())]:
        print(f'{x[0]} : {x[1]/1.e6} TWh, {round(100*x[1]/prod,3)} % ')
    
    
    tup = [(x[0],x[1]) for x in zip(B.index,B['Consommation'])]
    tup = sorted(tup, key=lambda x: x[1], reverse = True)
    print(f'En {year} la consommation maximale, en une heure, est de {tup[0][1]/1.e3 } GWh. Elle est advenue le {format_date(tup[0][0]) }')
    print(f'En {year} la consommation minimale, en une heure, est de {tup[-1][1]/1.e3 } GWh. Elle est advenue le {format_date(tup[-1][0]) }')
    return
    
def stat_prod_consommation(year):
    
    #Local import
    from .vre_eoles_read import read_rte
    
    # 3rd party import
    import numpy as np
    import matplotlib.pyplot as plt
    import matplotlib.ticker as ticker  
    
    cat = 'Consommation'
    B = read_rte(year)
    BN = B[cat].resample('M').sum()/1.e6

    fig=plt.figure(figsize=(15,15))
    plt.subplot(2,2,1)
    ax = BN.plot(kind='bar')
    #ticklabels = [item.strftime('%m') for item in B.index]
    ticklabels = range(1,13)
    ticklabels = ['jan','feb','mar','apr','may','jun','jul','aug','sep','oct','nov','dec']
    ax.xaxis.set_major_formatter(ticker.FixedFormatter(ticklabels))
    plt.gcf().autofmt_xdate()
    plt.ylabel("TWh")
    plt.title(f'{cat} ({year})')

    plt.subplot(2,2,2)
    plt.hist(B['Consommation']/1.e3,bins=30,color='g')
    plt.xlabel("GWh")
    plt.ylabel("# heures")
    
    plt.subplot(2,2,4)
    plt.boxplot(B['Consommation']/1.e3)
    plt.ylabel("GWh")
    plt.show()
    
    return

def prod_eolien_evolution_par_jour(year,month):
    '''
    Draw electricity production vs time
    cat must be equal to : 'Fioul', 'Charbon', 'Gaz', 'Nucléaire', 'Eolien',
           'Solaire', 'Hydraulique', 'Pompage', 'Bioénergies'

    '''
    
    #Local import
    from .vre_eoles_read import read_rte
    
    # 3rd party import
    import matplotlib.pyplot as plt
    
    cat ='Eolien'
    B = read_rte(year)
    
    def numberOfDays(y, m):
      leap = 0
      if y% 400 == 0:
         leap = 1
      elif y % 100 == 0:
         leap = 0
      elif y% 4 == 0:
         leap = 1
      if m==2:
         return 28 + leap
      list = [1,3,5,7,8,10,12]
      if m in list:
         return 31
      return 30


    fig=plt.figure(figsize=(15,5))
    plt.subplot(1,2,1)
    dep = str(year)+'-' + str(month) +'-01'
    fin = str(year)+'-' + str(month) +'-'+ str(numberOfDays(year, month))
    B[dep:fin][cat].plot(style='or',ms=2)
    B['mean'] = B[dep:fin][cat].mean()
    B[dep:fin]['mean'].plot()
    plt.ylabel("MWh")
    plt.title(f'(a) {cat}')

    plt.subplot(1,2,2)
    B[dep:fin][cat].diff().plot(style='or',ms=2)
    plt.ylabel("MWh/h")
    plt.title(f'(b) {cat}')
    plt.show()
    
    return



__all__ = [
    "request_vwf_wind",
    "request_vwf_pv",
    "vwf_api",
]


"""
Tranlation of the WinPower turbine names to the ninja ones .

"""

wp2wvf = {
    "Acciona AW-1500 77": "Acciona AW77 1500",  # ok
    "Alstom Power 110": "Alstom Eco 110",  # ok
    "Ecotecnia 80 1.6": "Alstom Eco 80",  # nok
    "Enercon E48 800": "Enercon E48 800",  # ok
    "Enercon E70 2000": "Enercon E70 2000",  # ok
    "Enercon E70 2300": "Enercon E70 2300",  # ok
    "Enercon E82 2000": "Enercon E82 2300",  # nok
    "Enercon E82 2300": "Enercon E82 2300",  # ok
    "Fuhrländer FL 2500 100": "Nordex N100 2500",  # ok
    "Fuhrländer FL 2500 90": "Nordex N80 2500",  # ok
    "GE Energy 1.5s": "GE 1.5sl",  # nok
    "Gamesa G80 2000": "Gamesa G80 2000",  # ok
    "Gamesa G90 2000": "Gamesa G90 2000",  # ok
    "Leitwind LTW77-1500": "Acciona AW77 1500",  # ok
    "Nordex N100 2500": "Nordex N100 2500",  # ok
    "Nordex N80 2500": "Nordex N80 2500",  # ok
    "Nordex N90 2300": "Nordex N90 2300",  # ok
    "Nordex N90 2500": "Nordex N90 2500",  # ok
    "Repower MM70": "REpower MM70 2000",  # ok
    "Repower MM82": "REpower MM82 2000",  # ok
    "Repower MM92": "REpower MM92 2000",  # ok
    "Senvion MM82 2050": "REpower MM82 2000",  # nok
    "Senvion MM92 2050": "REpower MM92 2000",  # nok
    "Siemens SWT-2.3-93": "Siemens SWT 2.3 101",  # nok
    "Siemens SWT-3.0-101": "Siemens SWT 3.0 101",  # ok
    "Vestas V100 1800": "Vestas V100 1800",  # ok
    "Vestas V100 2000": "Vestas V100 2000",  # ok
    "Vestas V112 3000": "Vestas V112 3000",  # ok
    "Vestas V52 850": "Vestas V52 850",  # ok
    "Vestas V66 1750": "Vestas V66 1750",  # ok
    "Vestas V80 2000": "Vestas V80 2000",  # ok
    "Vestas V90 2000": "Vestas V80 2000",  # nok
    "Vestas V90 3000": "Vestas V90 3000",  # ok
    "Winwind WWD-1-64": "Vestas V52 850",  # nok
    "Nordex N117 2400": "Nordex N90 2500",  # ok
    "Gamesa G97 2000": "Gamesa G90 2000",  # nok
    "Enercon E101 3050": "Enercon E101 3000",  # nok
    "Enercon E92 2350": "Enercon E92 2350",  # ok
    "Vestas V112 3300": "Vestas V112 3300",  # ok
    "GE Energy 2.75-100": "GE 2.75 103",  # nok
    "Vestas V110 2000": "Vestas V100 2000",  # ok
    "Vestas V117 3300": "Vestas V112 3300",  # ok
    "Senvion MM100": "REpower MM70 2000",  # ok
    "Vestas V126 3300": "Vestas V112 3300",  # ok
    "Enercon E82 2350": "Enercon E92 2350",  # ok
    "Enercon E115 3000": "Enercon E101 3000",  # ok
    "Enercon E103 2350": "Enercon E92 2350",  # ok
    "Siemens SWT-3.0-113": "Siemens SWT 3.0 101",  # ok
    "Senvion 3.0M122": "Vestas V112 3000",  # nok
    "Siemens SWT-3.2-113": "Siemens SWT 3.6 107",  # ok
    "Nordex N117 3000": "Nordex N131 3000",  # ok
    "Vestas V100 2200": "Vestas V100 2000",  # ok
    "Senvion 3.4M122 NES": "Vestas V112 3300",  # ok
    "Enercon E82 3000": "Enercon E82 3000",  # ok
    "Nordex #ND": "Nordex N131 3000",  # ok
    "Vestas V105 3600": "Vestas V112 3300",  # ok
    "Vestas V112 3450": "Vestas V112 3300",  # nok
    "Nordex N131": "Nordex N131 3000",  # ok
    "Vestas V126 3450": "Vestas V112 3300",  # nok
    "Vestas V136 3450": "Vestas V112 3300",  # nok
    "Vestas V117 3450": "Vestas V112 3300",  # nok
    "Ecotecnia 100": "Enercon E101 3000",  # nok
    "GE Energy 2.5xl": "Nordex N90 2500",  # nok
    "Goldwind S50 750": "Goldwind GW82 1500",  # nok
    "GE Energy 1.5sl": "GE 1.5sl",  # ok
    "Gamesa G114 2000": "Gamesa G90 2000",  # nok
    "DDIS DDIS60": "Enercon E48 800",  # nok
    "Enercon E53 800": "Enercon E53 800",  # ok
    "Enercon E82 2050": "Enercon E82 1800",  # nok
    "Siemens SWT-1.3-62": "Siemens SWT 1.3 62",  # ok
    "Bonus B62 1300": "Bonus B62 1300",  # ok
    "Enercon E66 2000": "Enercon E66 2000",  # ok
    "Nordex N60 1300": "Nordex N60 1300",  # ok
    "Repower MD77": "REPower MD77 1500",  # ok
    "WES WES250": "Vestas V29 225",  # nok
    "Winwind WWD-3-103": "Enercon E101 3000",
    "Siemens SWT-4.0-130": "Siemens SWT 4.0 130",
}


def request_vwf_wind(args, token):

    '''
    send a request to ninja api
    adapted from https://www.renewables.ninja/documentation/api/python-example
    
    Args
        args (dict): {"lat":Latitude
                      "lon"]:Longitude
                      "date_from": yyyy-mm-dd
                      "date_to": yyyy-mm-dd
                      "raw": "true"
                      "capacity":1.0
                      "height": hub height
                      "turbine": turbine type
                      "format": "json"
                      
        token (string) : your ninja API token
    
    returns:
        data (dataframe): date| wind speed [ FC
        
        metadata (dataframe): conains the args values
        
        status (boolean): True theexchangz with ninja has been successfull False otherwise
        
    '''

    # Standard library imports
    import requests
    import json as json

    # 3rd party import
    import pandas as pd

    
    api_base = "https://www.renewables.ninja/api/"

    data = None
    metadata = None
    status = True

    s = requests.session()

    # Send token header with each request
    s.headers = {"Authorization": "Token " + token}
    url = api_base + "data/wind"

    r = s.get(url, params=args)

    try:
        parsed_response = json.loads(r.text)
        data = pd.read_json(json.dumps(parsed_response["data"]), orient="index")
        metadata = parsed_response["metadata"]
    except:
        status = False

    return data, metadata, status
    
def request_vwf_pv(args, token):

    '''send a request to ninja api
    adapted from https://www.renewables.ninja/documentation/api/python-example
    '''
    
    # Standard library imports
    import requests
    import json as json

    # 3rd party import
    import pandas as pd

    api_base = 'https://www.renewables.ninja/api/'

    s = requests.session()
    # Send token header with each request
    s.headers = {'Authorization': 'Token ' + token}

    url = api_base + 'data/pv'

    args = {
        'lat': 34.125,
        'lon': 39.814,
        'date_from': '2015-01-01',
        'date_to': '2015-12-31',
        'dataset': 'merra2',
        'capacity': 1.0,
        'system_loss': 0.1,
        'tracking': 0,
        'tilt': 35,
        'azim': 180,
        'format': 'json'
    }

    r = s.get(url, params=args)

    # Parse JSON to get a pandas.DataFrame of data and dict of metadata
    parsed_response = json.loads(r.text)

    data = pd.read_json(json.dumps(parsed_response['data']), orient='index')
    metadata = parsed_response['metadata']
    
    return data, metadata, status


def vwf_api(df, year, root_outputs, token, median_hub_height=80, EOLES=False):

    """
    This function automates the access to  ninja via its API taking into account ninja access restrition:

        - Anonymous users are limited to a maximum of 5 requests per day.

        - To increase this limit to 50 per hour, please register for a free user account.

        - In addition to the hourly limit, there is a burst limit, currently set to 6/minute for both anonymous and registered users, to prevent a fast succession of individual requests overloading the server.

        - If you need a higher limit, e.g. for automated access via the API, please contact ninja webmaster.

    Arguments:
        df (dataframe): index Insee county code|Latitude (°)|Longitue (°)|Total power (kW)|Manufacture|Model|Hub height (m)
        year (int): year of the analysis
        
        root_outputs (path): path of outputs directory
        
        token (string): your ninja API token (see https://www.renewables.ninja/profile )
        
        median_hub_height (real): height to be used if the hub height is unknown
        
    Returns:
        files: stored in root_outputs as:
            ##.xlsx where ## stands for the county Insee code with Timedate|electricity|wind_speed|
            ##_metadata.xls is the metadata returns by the ninja api

    """

    # Standard library imports
    import os
    import re
    import time
    from pathlib import Path
    
    # 3rd party imports
    import vre_eoles as vre
    import pandas as pd

    test = False

    def turbine_name(county,):

        try:
            turbine_type = (
                df.loc[county, "Manufacturer"]
                + " "
                + str(df.loc[county, "Model"].replace("/", " "))
            )
        except:
            turbine_type = (
                df.loc[county, "Manufacturer"] + " " + str(df.loc[county, "Model"])
            )
        turbine_name = wp2wvf[turbine_type]

        return turbine_name

    # build the list of existing file name ##.xlsx ## is county identifier
    files = []
    for file in os.listdir(root_outputs):
        sub_str = re.findall("\d{1,3}.xlsx", file)
        if len(sub_str):
            files.append(sub_str[0])

    turbine_type = []
    list_args = {}

    idx = 0
    for county in df.index:
        if idx == 49:
            break  # to comply with the request hour limit
        args = {}  # dict to provide to the ninja API

        args["lat"] = df.loc[county, "Latitude"]
        args["lon"] = df.loc[county, "Longitude"]
        args["date_from"] = str(year) + "-01-01"
        args["date_to"] = str(year) + "-12-31"
        args["raw"] = "true"
        args["capacity"] = 1.0
        if EOLES:
            args["height"] = 80
            args["turbine"] = "Vestas V90 2000"
        else:
            height = df.loc[county, "Hub height"]
            if height == "#ND":
                height = median_hub_height  # if the height is unknown we take the median value
            # of the known heights
            args["height"] = height
            args["turbine"] = turbine_name(county)
        args["format"] = "json"
        list_args[county] = args

        if (not test) & (not (county + ".xlsx" in files)):
            print(county + ".xlsx", end=" ", flush=True)
            idx += 1
            data, metadata, status = request_vwf_wind(args, token)
            if (
                status
            ):  # if the request succeeds we save the results into two excel files
                data.to_excel(root_outputs / Path(str(county) + ".xlsx"))
                meta_data = pd.DataFrame.from_dict(metadata).T
                meta_data.to_excel(root_outputs / Path(str(county) + "_metadata.xlsx"))
            else:
                print("ERREUR :", county)
            time.sleep(11)  # to comply with the burst limit



#!/usr/bin/env python

from setuptools import setup, find_packages

# read the contents of your README file
from os import path
this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

#requires = open(file_requirements).read().strip().split('\n')
    
# This setup is suitable for "python setup.py develop".

setup(name='vre_eoles',
      version='0.2.0',
      description='toolbox for computing charge factor used in EOLES model',
      long_description=long_description,
      long_description_content_type='text/markdown',
      include_package_data = True,
      license = 'MIT',
      classifiers = [
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Topic :: Scientific/Engineering :: Physics',
        'Operating System :: OS Independent',
        'Intended Audience :: Science/Research'
        ],
      keywords = 'Image, data processing',
      install_requires = ['pandas',
                          'numpy',
                          'scipy',
                          'matplotlib',
                          'sklearn',
                          'adjustText'],
      author= 'ArrayStream(François Bertin)',
      author_email= 'francois.bertin7@wanadoo.fr',
      url= 'https://github.com/Bertin-fap/CIRED-ENS',
      packages=find_packages(),
      )



__version__ = "0.2.0"
__author__ = 'F. Bertin'
__license__ = "MIT"

from .vre_eoles_ninja import *
from .vre_eoles_math import *
from .vre_eoles_read import *
from .vre_eoles_rte import *
from .vre_eoles_wind import *





__all__ = [
    "read_LF_CIRED",
    "read_parc_eolien",
    "read_rte",
    "read_parc_solaire",
    "read_wind_power_power_curve",
    "read_LF_onshore",
    "read_vwf_power_curves",
]


def read_LF_CIRED(year, technic, directory_inputs):
    """
    reading of  an excel EOLES file
    
    Args:
    year (int): year must be in [2006,2012,...,2017]
        
    Returns:
        fe (dataframe): {year:nominal_pwer(GW)}
    """

    # Standard library import
    from pathlib import Path
    import datetime

    # 3rd party import
    import pandas as pd

    fe = pd.read_csv(
        directory_inputs / Path("vre_profiles" + str(year) + ".csv"), header=None
    )
    deb = datetime.datetime(year, 1, 1)
    fe.rename(columns={0: "Technic", 1: "Hour", 2: str(year)}, inplace=True)
    fe.query("Technic==@technic", inplace=True)
    fe.index = [deb + datetime.timedelta(hours=i) for i in range(len(fe))]
    fe.drop(["Technic", "Hour"], axis=1, inplace=True)

    return fe


def read_parc_eolien():

    """
    reading of  an excel RTE file downloaded from:
    https://www.statistiques.developpement-durable.gouv.fr/publicationweb/262
    
    Args:
        
    Returns:
        Pow_eol (dict): {year:nominal_pwer(GW)}
    """

    # Standard library import
    from pathlib import Path
    import os

    # 3rd party import
    import pandas as pd

    directory_eol = Path.home() / Path("CIRED-ENS")
    path = directory_eol / Path("Évolution du parc éolien.csv")

    Pow_eol = pd.read_csv(
        path, sep=";", parse_dates=["Date"], usecols=["Date", "Terrestre"]
    )
    Pow_eol.set_index("Date", inplace=True)
    Pow_eol = Pow_eol.dropna()
    Pow_eol = Pow_eol.resample("Y").mean()
    Pow_eol.index = Pow_eol.index.year
    Pow_eol = Pow_eol["Terrestre"].to_dict()

    return Pow_eol


def read_rte(year):
    """
    reading of  an excel RTE file downloaded from https://www.rte-france.com/eco2mix/telecharger-les-indicateurs
    Unzip the file and remame then yyyy.xlsx where yyyy stands for the 4 digits of the year.
    
    Args:
        year (int) : year of the RTE excel file to be read
    Returns:
        B (dataframe): datafame of the RTE file
    """

    # Standard library import
    from pathlib import Path
    import os

    # 3rd party import
    import pandas as pd

    directory_rte = Path.home() / Path("FRENCH_STAT/RTE_DATA")
    path = directory_rte / Path(str(year) + ".xlsx")

    A = pd.read_excel(
        path,
        skipfooter=1,  # skip the last line which is a comment
        parse_dates=[
            ["Date", "Heures"]
        ],  # concanenate columns Date and Heures as TimeStamp
        usecols="c:E,H:Q",
    )  # select columns
    A.drop([i for i in list(A.index)[1::2]], inplace=True)  # drop irrelevent rows
    index_ = [
        x for i, x in enumerate(A["Date_Heures"]) if i % 2 == 0
    ]  # resample every hour and convert in energy
    A.drop(columns=["Date_Heures"], inplace=True)
    B = A.groupby(A.index // 4).mean()
    B["Date_Heures"] = index_
    B.set_index("Date_Heures", inplace=True)
    return B


def read_parc_solaire():

    """
    WEB scrapping ot the wiki page : https://fr.wikipedia.org/wiki/%C3%89nergie_solaire_en_France and extract
    the table : Puissance photovoltaïque par région (MW)
    Valid 30/12/2020
    
    Args:
        
    Returns:
        Pow_sol (dict): {year:nominal_pwer(MW)}
    """

    # 3rd party imports
    import pandas as pd

    url = "https://fr.wikipedia.org/wiki/%C3%89nergie_solaire_en_France"

    tables = pd.read_html(url, header=0)

    headings = set(
        [
            "Région",
            "2008",
            "2009",
            "2010",
            "2011",
            "2012",
            "2013",
            "2014",
            "2015",
            "2016",
            "2017",
            "2018",
        ]
    )  # must be updated with wiki upgrade

    for (
        table
    ) in tables:  # find the table named "Puissance photovoltaïque par région (MW)"
        if set(table.columns.values) == headings:
            break

    Pow_sol = table.query('Région=="Total France"').to_dict("records")[0]
    del Pow_sol["Région"]
    Pow_sol = {
        int(key): int(str(value).replace("\xa0", "")) for key, value in Pow_sol.items()
    }  # data cleaning
    Pow_sol[2012] = 3727
    Pow_sol[2013] = 4366
    Pow_sol[
        2019
    ] = 9900  # https://www.statistiques.developpement-durable.gouv.fr/tableau-de-bord-solaire-photovoltaique-quatrieme-trimestre-2019
    return Pow_sol


def read_LF_onshore(year, directory_inputs):
    """
    reading of  an excel EOLES file
    
    Args:
        year (int): year must be in [2006,2012,...,2017]
        directory_inputs (string): directory containing the vre_path_profiles
        
    Returns:
        Pow_eol (dict): {year:nominal_pwer(GW)}
    """

    # Standard library imports
    from pathlib import Path
    import datetime

    # 3rd party imports
    import pandas as pd

    fe = pd.read_csv(
        directory_inputs / Path("vre_profiles" + str(year) + ".csv"), header=None
    )
    deb = datetime.datetime(2006, 1, 1)
    temps = [deb + datetime.timedelta(hours=i) for i in range(int(len(fe) / 3))]
    fe.rename(columns={0: "Technic", 1: "Hour", 2: str(year)}, inplace=True)
    fe.query('Technic=="onshore"', inplace=True)
    fe.index = temps
    fe.drop(["Technic", "Hour"], axis=1, inplace=True)

    return fe

def read_vwf_power_curves(root_inputs_vwf):

    # Standard library imports
    from pathlib import Path

    # 3rd party imports
    import pandas as pd

    Power_cuve00 = pd.read_csv(root_inputs_vwf / Path('Wind Turbine Power Curves ~ 5 (0.01ms with 0.00 w smoother).csv'),
                           index_col="speed")
    Power_cuve01 = pd.read_csv(root_inputs_vwf / Path('Wind Turbine Power Curves ~ 5 (0.01ms with 0.10 w smoother).csv'),
                           index_col="data$speed")
    Power_cuve02 = pd.read_csv(root_inputs_vwf / Path('Wind Turbine Power Curves ~ 5 (0.01ms with 0.20 w smoother).csv'),
                           index_col="data$speed")
    Power_cuve03 = pd.read_csv(root_inputs_vwf / Path('Wind Turbine Power Curves ~ 5 (0.01ms with 0.30 w smoother).csv'),
                           index_col="data$speed")
    Power_cuve04 = pd.read_csv(root_inputs_vwf / Path('Wind Turbine Power Curves ~ 5 (0.01ms with 0.40 w smoother).csv'),
                           index_col="data$speed")
                           
    return Power_cuve00, Power_cuve01, Power_cuve02, Power_cuve03, Power_cuve04
    
def read_wind_power_power_curve(constructor, turbine_type, file):

    """Returns the power curve of the WindPower Power_curves_20201224.xls database
    interpolated from 0 to 35 m/s with a step of 0.01 m/s. (vwf convention)
    
    Args:
        constructor (string): turbine manufacturer (ex: "Vestas")
        turbine_type (string): turbine model (ex:"V90/2000") 
        file (string): full path of the file Power_curves_20201224.xls
        
    Returns:
        wind_speed_interp (nparray): wind speed [0, 0.01, 0.02,...,35] in m/s
        pow_interp (nparray): turbine power at wind_speed_interp values
    """

    # Standard library imports
    from pathlib import Path
    
    # 3rd party import
    import pandas as pd
    import numpy as np
    from scipy.interpolate import interp1d

    dwp = pd.read_excel(file, sheet_name="Power_curves")
    rep = dwp.query(
        "(`Manufucturer Name`==@constructor) & (`Turbine Name`==@turbine_type)"
    )
    wind_speed = np.array(dwp.iloc[0][4:-2]).astype(float)
    power = np.array(rep.iloc[0][4:-2]).astype(float)

    #  interpolation
    wind_speed_interp = np.arange(0, 35.01, 0.01)
    f = interp1d(wind_speed, power, kind="linear")
    pow_interp = f(wind_speed_interp)

    return wind_speed_interp, pow_interp